@@ -254,7 +254,11 @@ def process_signature(
 
     obj = inspect.unwrap(obj)
     sph_signature = sphinx_signature(obj)
-    parameters = [param.replace(annotation=inspect.Parameter.empty) for param in sph_signature.parameters.values()]
+
+    if app.config.typehints_use_signature:
+        parameters = list(sph_signature.parameters.values())
+    else:
+        parameters = [param.replace(annotation=inspect.Parameter.empty) for param in sph_signature.parameters.values()]
 
     # if we have parameters we may need to delete first argument that's not documented, e.g. self
     start = 0
@@ -277,7 +281,10 @@ def process_signature(
             if not isinstance(method_object, (classmethod, staticmethod)):
                 start = 1
 
-    sph_signature = sph_signature.replace(parameters=parameters[start:], return_annotation=inspect.Signature.empty)
+    sph_signature = sph_signature.replace(parameters=parameters[start:])
+    if not app.config.typehints_use_signature_return:
+        sph_signature = sph_signature.replace(return_annotation=inspect.Signature.empty)
+
     return stringify_signature(sph_signature).replace("\\", "\\\\"), None
 
 
@@ -468,19 +475,27 @@ def split_type_comment_args(comment: str) -> list[str | None]:
     return result
 
 
-def format_default(app: Sphinx, default: Any) -> str | None:
+def format_default(app: Sphinx, default: Any, is_annotated: bool) -> str | None:
     if default is inspect.Parameter.empty:
         return None
     formatted = repr(default).replace("\\", "\\\\")
-    if app.config.typehints_defaults.startswith("braces"):
-        return f" (default: ``{formatted}``)"
+
+    if is_annotated:
+        if app.config.typehints_defaults.startswith("braces"):
+            return f" (default: ``{formatted}``)"
+        else:  # other option is comma
+            return f", default: ``{formatted}``"
     else:
-        return f", default: ``{formatted}``"
+        if app.config.typehints_defaults == "braces-after":
+            return f" (default: ``{formatted}``)"
+        else:
+            return f"default: ``{formatted}``"
 
 
 def process_docstring(
     app: Sphinx, what: str, name: str, obj: Any, options: Options | None, lines: list[str]  # noqa: U100
 ) -> None:
+
     original_obj = obj
     obj = obj.fget if isinstance(obj, property) else obj
     if not callable(obj):
@@ -519,6 +534,8 @@ def _get_sphinx_line_keyword_and_argument(line: str) -> tuple[str, str | None] |
 
     split_directive_and_name = param_line_without_description[1].split(maxsplit=1)  # noqa: SC200
     if len(split_directive_and_name) != 2:
+        if not len(split_directive_and_name):
+            return None
         return (split_directive_and_name[0], None)
 
     return tuple(split_directive_and_name)  # type: ignore
@@ -552,41 +569,45 @@ def _inject_types_to_docstring(
     name: str,
     lines: list[str],
 ) -> None:
-    for arg_name, annotation in type_hints.items():
-        if arg_name == "return":
-            continue  # this is handled separately later
-        if signature is None or arg_name not in signature.parameters:
-            default = inspect.Parameter.empty
-        else:
+
+    if signature is not None:
+        for arg_name in signature.parameters:
+            annotation = type_hints.get(arg_name, None)
+
             default = signature.parameters[arg_name].default
-        if arg_name.endswith("_"):
-            arg_name = f"{arg_name[:-1]}\\_"
 
-        formatted_annotation = format_annotation(annotation, app.config)
+            if arg_name.endswith("_"):
+                arg_name = f"{arg_name[:-1]}\\_"
 
-        insert_index = None
-        for at, line in enumerate(lines):
-            if _line_is_param_line_for_arg(line, arg_name):
-                # Get the arg_name from the doc to match up for type in case it has a star prefix.
-                # Line is in the correct format so this is guaranteed to return tuple[str, str].
-                _, arg_name = _get_sphinx_line_keyword_and_argument(line)  # type: ignore[assignment, misc]
-                insert_index = at
-                break
+            insert_index = None
+            for at, line in enumerate(lines):
+                if _line_is_param_line_for_arg(line, arg_name):
+                    # Get the arg_name from the doc to match up for type in case it has a star prefix.
+                    # Line is in the correct format so this is guaranteed to return tuple[str, str].
+                    _, arg_name = _get_sphinx_line_keyword_and_argument(line)  # type: ignore[assignment, misc]
+                    insert_index = at
+                    break
 
-        if insert_index is None and app.config.always_document_param_types:
-            lines.append(f":param {arg_name}:")
-            insert_index = len(lines)
+            if annotation is not None and insert_index is None and app.config.always_document_param_types:
+                lines.append(f":param {arg_name}:")
+                insert_index = len(lines)
 
-        if insert_index is not None:
-            type_annotation = f":type {arg_name}: {formatted_annotation}"
-            if app.config.typehints_defaults:
-                formatted_default = format_default(app, default)
-                if formatted_default:
-                    if app.config.typehints_defaults.endswith("after"):
-                        lines[insert_index] += formatted_default
-                    else:  # add to last param doc line
-                        type_annotation += formatted_default
-            lines.insert(insert_index, type_annotation)
+            if insert_index is not None:
+                if annotation is None:
+                    type_annotation = f":type {arg_name}: "
+                else:
+                    formatted_annotation = format_annotation(annotation, app.config)
+                    type_annotation = f":type {arg_name}: {formatted_annotation}"
+
+                if app.config.typehints_defaults:
+                    formatted_default = format_default(app, default, annotation is not None)
+                    if formatted_default:
+                        if app.config.typehints_defaults.endswith("after"):
+                            lines[insert_index] += formatted_default
+                        else:  # add to last param doc line
+                            type_annotation += formatted_default
+
+                lines.insert(insert_index, type_annotation)
 
     if "return" in type_hints and not inspect.isclass(original_obj):
         if what == "method" and name.endswith(".__init__"):  # avoid adding a return type for data class __init__
@@ -632,6 +653,8 @@ def setup(app: Sphinx) -> dict[str, bool]:
     app.add_config_value("typehints_defaults", None, "env")
     app.add_config_value("simplify_optional_unions", True, "env")
     app.add_config_value("typehints_formatter", None, "env")
+    app.add_config_value("typehints_use_signature", False, "env")
+    app.add_config_value("typehints_use_signature_return", False, "env")
     app.connect("env-before-read-docs", validate_config)  # config may be changed after “config-inited” event
     app.connect("autodoc-process-signature", process_signature)
     app.connect("autodoc-process-docstring", process_docstring)
